{% extends "email_layout.html" %}

{% block content %}
<p>{{username}} mentioned you in a comment to the following article: <a href="{{link}}">{{link}}</a></p>
<p>Comment: </p>
<p>{{text}}</p>
{% endblock %}
